﻿using UnityEngine;
using System.Collections;
using System.Collections.Generic;

public class Tile : MonoBehaviour {

	public List<GameObject> neighbourTiles = new List<GameObject>();
	private int zoning = 0;
	private int tier = 0;

	GameObject marker;

	void Start () {
		GetComponent<SpriteRenderer> ().sprite = GameLogic.Instance.grass;
		Invoke ("GetNeighbourTiles", 0.5f);
		InvokeRepeating ("Sprawl", Random.Range(GameLogic.Instance.waveCooldownTime, GameLogic.Instance.waveCooldownTime + 0.1f), GameLogic.Instance.waveCooldownTime);
	}

	void Update(){
		if (GameLogic.Instance.GameState == GameLogic.GAMESTATE_PLAYING) {
		}
	}


	void GetNeighbourTiles(){
		Vector3 neighbourPlace = new Vector3 (transform.position.x + 1.5f, transform.position.y - 0.5f);
		RaycastHit2D tileHit = Physics2D.Raycast (neighbourPlace, Vector2.zero);
		if (tileHit != null && tileHit.collider != null) {
			neighbourTiles.Add(tileHit.collider.gameObject);
		}
		neighbourPlace = new Vector3 (transform.position.x + 0.5f, transform.position.y - 1.5f);
		tileHit = Physics2D.Raycast (neighbourPlace, Vector2.zero);
		if (tileHit != null && tileHit.collider != null) {
			neighbourTiles.Add(tileHit.collider.gameObject);
		}
		neighbourPlace = new Vector3 (transform.position.x - 0.5f, transform.position.y - 0.5f);
		tileHit = Physics2D.Raycast (neighbourPlace, Vector2.zero);
		if (tileHit != null && tileHit.collider != null) {
			neighbourTiles.Add(tileHit.collider.gameObject);
		}
		neighbourPlace = new Vector3 (transform.position.x + 0.5f, transform.position.y + 0.5f);
		tileHit = Physics2D.Raycast (neighbourPlace, Vector2.zero);
		if (tileHit != null && tileHit.collider != null) {
			neighbourTiles.Add(tileHit.collider.gameObject);
		}
	}

	public void IncreaseZoning(){
		if (zoning == 0) {
			marker = (GameObject)Instantiate (GameLogic.Instance.clickMarker, gameObject.transform.position, Quaternion.identity, gameObject.transform);
		}
		zoning += 3;
	}

	public void DecreaseZoning(){
		zoning--;
		if (zoning < 0) {
			zoning = 0;
			Destroy (marker);
		}
	}

	public int GetZoning(){
		return zoning;
	}

	public int GetTier(){
		return tier;
	}

	public void SetTier(int newTier){
		int oldTier = tier;
		tier = newTier;
		if (tier == 0) {
			GetComponent<SpriteRenderer> ().sprite = GameLogic.Instance.grass;
		} else if (tier == 1) {
			GetComponent<SpriteRenderer> ().sprite = GameLogic.Instance.building1;
		} else if (tier == 2) {
			GetComponent<SpriteRenderer> ().sprite = GameLogic.Instance.building2;
		} else if (tier == 3) {
			GetComponent<SpriteRenderer> ().sprite = GameLogic.Instance.building3;
		}
		GameLogic.Instance.population += newTier - oldTier;
		if (GameLogic.Instance.population < 0) {
			GameLogic.Instance.population = 0;
		}
	}

	void Sprawl(){
		if (GameLogic.Instance.GameState == GameLogic.GAMESTATE_PLAYING) {
			if (tier > 0) {
				int pressure = 0;
				foreach (GameObject neighbourTile in neighbourTiles) {
					pressure += neighbourTile.GetComponent<Tile> ().GetTier ();
				}

				pressure += GetZoning ();

				if (pressure > 7 && GetTier () < 3 && GameLogic.Instance.freePopulation >= 3 - GetTier ()) {
					GameLogic.Instance.freePopulation -= 3 - GetTier ();
					SetTier (3);
				} else if (pressure > 3 && GetTier () < 2 && GameLogic.Instance.freePopulation >= 2 - GetTier ()) {
					GameLogic.Instance.freePopulation -= 2 - GetTier ();
					SetTier (2);
				}

				if (GameLogic.Instance.freePopulation <= 1 && pressure > 4) {
					GameLogic.Instance.abandon = true;
				}

				if (GameLogic.Instance.abandon == true && pressure <= (2 + Random.Range (0, 6)) && GetTier () > 0) {
					SetTier (GetTier () - 1);
					GameLogic.Instance.freePopulation += 1;
					GameLogic.Instance.abandon = false;
				}
			} else {
				int pressure = 0;
				foreach (GameObject neighbourTile in neighbourTiles) {
					pressure += neighbourTile.GetComponent<Tile> ().GetTier ();
				}

				pressure += GetZoning ();

				if (pressure >= 2 && GetTier () < 1 && GameLogic.Instance.freePopulation > 0) {
					GameLogic.Instance.freePopulation -= 1 - GetTier ();
					SetTier (1);
				} else if (GameLogic.Instance.freePopulation > 3) {
					GameLogic.Instance.freePopulation -= 1 - GetTier ();
					SetTier (1);
				}
			}
			DecreaseZoning ();
		}
	}
}
